/*******************************************************************************
 *
 * Copyright (C) 2017 Xilinx, Inc.  All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Use of the Software is limited solely to applications:
 * (a) running on a Xilinx device, or
 * (b) that interact with a Xilinx device through a bus or interconnect.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of the Xilinx shall not be used
 * in advertising or otherwise to promote the sale, use or other dealings in
 * this Software without prior written authorization from Xilinx.
 *
*******************************************************************************/
/******************************************************************************/
/**
 *
 * @file xvidc.c
 * @addtogroup video_common_v4_3
 * @{
 *
 * Contains common utility functions that are typically used by video-related
 * drivers and applications.
 *
 * @note	None.
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -----------------------------------------------
 * 1.0   rc,  01/10/15 Initial release.
 *       als
 * 2.2   als  02/01/16 Functions with pointer arguments that don't modify
 *                     contents now const.
 *                     Added ability to insert a custom video timing table.
 *       yh            Added 3D support.
 * 3.0   aad  05/13/16 Added API to search for RB video modes.
 * 3.1   rco  07/26/16 Added extern definition for timing table array
 *                     Added video-in-memory color formats
 *                     Updated XVidC_RegisterCustomTimingModes API signature
 * 4.1   rco  11/23/16 Added new memory formats
 *                     Added new API to get video mode id that matches exactly
 *                     with provided timing information
 *                     Fix c++ warnings
 * 4.2	 jsr  07/22/17 Added new framerates and color formats to support SDI
 *                     Reordered YCBCR422 colorforamt and removed other formats
 *                     that are not needed for SDI which were added earlier.
 *       vyc  10/04/17 Added new streaming alpha formats and new memory formats
 * 4.3   eb   26/01/18 Added API XVidC_GetVideoModeIdExtensive
 *       jsr  02/22/18 Added XVIDC_CSF_YCBCR_420 color space format
 *       vyc  04/04/18 Added BGR8 memory format
 * </pre>
 *
*******************************************************************************/

/******************************* Include Files ********************************/

#include "xil_assert.h"
#include "xstatus.h"
#include "xvidc.h"
#include "debug.h"
/*************************** Variable Declarations ****************************/
const XVidC_VideoTimingMode XVidC_VideoTimingModes[XVIDC_VM_NUM_SUPPORTED] =
{
	/* Interlaced modes. */
	{ XVIDC_VM_720x480_60_I, "720x480@60Hz (I)", XVIDC_FR_60HZ,
		{720, 19, 62, 57, 858, 0,
		240, 4, 3, 15, 262, 5, 3, 15, 263, 0} },
	{ XVIDC_VM_720x576_50_I, "720x576@50Hz (I)", XVIDC_FR_50HZ,
		{720, 12, 63, 69, 864, 0,
		288, 2, 3, 19, 312, 3, 3, 19, 313, 0} },
	{ XVIDC_VM_1440x480_60_I, "1440x480@60Hz (I)", XVIDC_FR_60HZ,
		{1440, 38, 124, 114, 1716, 0,
		240, 4, 3, 15, 262, 5, 3, 15, 263, 0} },
	{ XVIDC_VM_1440x576_50_I, "1440x576@50Hz (I)", XVIDC_FR_50HZ,
		{1440, 24, 126, 138, 1728, 0,
		288, 2, 3, 19, 312, 3, 3, 19, 313, 0} },
	{ XVIDC_VM_1920x1080_48_I, "1920x1080@48Hz (I)", XVIDC_FR_48HZ,
		{1920, 371, 88, 371, 2750, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_1920x1080_50_I, "1920x1080@50Hz (I)", XVIDC_FR_50HZ,
		{1920, 528, 44, 148, 2640, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_1920x1080_60_I, "1920x1080@60Hz (I)", XVIDC_FR_60HZ,
		{1920, 88, 44, 148, 2200, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_1920x1080_96_I, "1920x1080@96Hz (I)", XVIDC_FR_96HZ,
		{1920, 371, 88, 371, 2750, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },
	{ XVIDC_VM_1920x1080_100_I, "1920x1080@100Hz (I)", XVIDC_FR_100HZ,
		{1920, 528, 44, 148, 2640, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },
	{ XVIDC_VM_1920x1080_120_I, "1920x1080@120Hz (I)", XVIDC_FR_120HZ,
		{1920, 88, 44, 148, 2200, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },
	{ XVIDC_VM_2048x1080_48_I, "2048x1080@48Hz (I)", XVIDC_FR_48HZ,
		{2048, 329, 44, 329, 2750, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_2048x1080_50_I, "2048x1080@50Hz (I)", XVIDC_FR_50HZ,
		{2048, 274, 44, 274, 2640, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_2048x1080_60_I, "2048x1080@60Hz (I)", XVIDC_FR_60HZ,
		{2048, 66, 20, 66, 2200, 1,
		540, 2, 5, 15, 562, 3, 5, 15, 563, 1} },
	{ XVIDC_VM_2048x1080_96_I, "2048x1080@96Hz (I)", XVIDC_FR_96HZ,
		{2048, 329, 44, 329, 2750, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },
	{ XVIDC_VM_2048x1080_100_I, "2048x1080@100Hz (I)", XVIDC_FR_100HZ,
		{2048, 274, 44, 274, 2640, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },
	{ XVIDC_VM_2048x1080_120_I, "2048x1080@120Hz (I)", XVIDC_FR_120HZ,
		{2048, 66, 20, 66, 2200, 1,
		1080, 4, 10, 30, 1124, 6, 10, 30, 1126, 1} },


	/* Progressive modes. */
	{ XVIDC_VM_640x350_85_P, "640x350@85Hz", XVIDC_FR_85HZ,
		{640, 32, 64, 96, 832, 1,
		350, 32, 3, 60, 445, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_640x480_60_P, "640x480@60Hz", XVIDC_FR_60HZ,
		{640, 8+8, 96, 40+8, 800, 0,
		480, 2+8, 2, 25+8, 525, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_640x480_72_P, "640x480@72Hz", XVIDC_FR_72HZ,
		{640, 8+16, 40, 120+8, 832, 0,
		480, 8+1, 3, 20+8, 520, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_640x480_75_P, "640x480@75Hz", XVIDC_FR_75HZ,
		{640, 16, 64, 120, 840, 0,
		480, 1, 3, 16, 500, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_640x480_85_P, "640x480@85Hz", XVIDC_FR_85HZ,
		{640, 56, 56, 80, 832, 0,
		480, 1, 3, 25, 509, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_720x400_85_P, "720x400@85Hz", XVIDC_FR_85HZ,
		{720, 36, 72, 108, 936, 0,
		400, 1, 3, 42, 446, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_720x480_60_P, "720x480@60Hz", XVIDC_FR_60HZ,
		{720, 16, 62, 60, 858, 0,
		480, 9, 6, 30, 525, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_720x576_50_P, "720x576@50Hz", XVIDC_FR_50HZ,
		{720, 12, 64, 68, 864, 0,
		576, 5, 5, 39, 625, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_800x600_56_P, "800x600@56Hz", XVIDC_FR_56HZ,
		{800, 24, 72, 128, 1024, 1,
		600, 1, 2, 22, 625, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_800x600_60_P, "800x600@60Hz", XVIDC_FR_60HZ,
		{800, 40, 128, 88, 1056, 1,
		600, 1, 4, 23, 628, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_800x600_72_P, "800x600@72Hz", XVIDC_FR_72HZ,
		{800, 56, 120, 64, 1040, 1,
		600, 37, 6, 23, 666, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_800x600_75_P, "800x600@75Hz", XVIDC_FR_75HZ,
		{800, 16, 80, 160, 1056, 1,
		600, 1, 3, 21, 625, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_800x600_85_P, "800x600@85Hz", XVIDC_FR_85HZ,
		{800, 32, 64, 152, 1048, 1,
		600, 1, 3, 27, 631, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_800x600_120_P_RB, "800x600@120Hz (RB)", XVIDC_FR_120HZ,
		{800, 48, 32, 80, 960, 1,
		600, 3, 4, 29, 636, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_848x480_60_P, "848x480@60Hz", XVIDC_FR_60HZ,
		{848, 16, 112, 112, 1088, 1,
		480, 6, 8, 23, 517, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1024x768_60_P, "1024x768@60Hz", XVIDC_FR_60HZ,
		{1024, 24, 136, 160, 1344, 0,
		768, 3, 6, 29, 806, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1024x768_70_P, "1024x768@70Hz", XVIDC_FR_70HZ,
		{1024, 24, 136, 144, 1328, 0,
		768, 3, 6, 29, 806, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1024x768_75_P, "1024x768@75Hz", XVIDC_FR_75HZ,
		{1024, 16, 96, 176, 1312, 1,
		768, 1, 3, 28, 800, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1024x768_85_P, "1024x768@85Hz", XVIDC_FR_85HZ,
		{1024, 48, 96, 208, 1376, 1,
		768, 1, 3, 36, 808, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1024x768_120_P_RB, "1024x768@120Hz (RB)", XVIDC_FR_120HZ,
		{1024, 48, 32, 80, 1184, 1,
		768, 3, 4, 38, 813, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1152x864_75_P, "1152x864@75Hz", XVIDC_FR_75HZ,
		{1152, 64, 128, 256, 1600, 1,
		864, 1, 3, 32, 900, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x720_24_P, "1280x720@24Hz", XVIDC_FR_24HZ,
		{1280, 1760, 40, 220, 3300, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x720_25_P, "1280x720@25Hz", XVIDC_FR_25HZ,
		{1280, 2420, 40, 220, 3960, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x720_30_P, "1280x720@30Hz", XVIDC_FR_30HZ,
		{1280, 1760, 40, 220, 3300, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x720_50_P, "1280x720@50Hz", XVIDC_FR_50HZ,
		{1280, 440, 40, 220, 1980, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x720_60_P, "1280x720@60Hz", XVIDC_FR_60HZ,
		{1280, 110, 40, 220, 1650, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x768_60_P, "1280x768@60Hz", XVIDC_FR_60HZ,
		{1280, 64, 128, 192, 1664, 0,
		768, 3, 7, 20, 798, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x768_60_P_RB, "1280x768@60Hz (RB)", XVIDC_FR_60HZ,
		{1280, 48, 32, 80, 1440, 1,
		768, 3, 7, 12, 790, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1280x768_75_P, "1280x768@75Hz", XVIDC_FR_75HZ,
		{1280, 80, 128, 208, 1696, 0,
		768, 3, 7, 27, 805, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x768_85_P, "1280x768@85Hz", XVIDC_FR_85HZ,
		{1280, 80, 136, 216, 1712, 0,
		768, 3, 7, 31, 809, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x768_120_P_RB, "1280x768@120Hz (RB)", XVIDC_FR_120HZ,
		{1280, 48, 32, 80, 1440, 1,
		768, 3, 7, 35, 813, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1280x800_60_P, "1280x800@60Hz", XVIDC_FR_60HZ,
		{1280, 72, 128, 200, 1680, 0,
		800, 3, 6, 22, 831, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x800_60_P_RB, "1280x800@60Hz (RB)", XVIDC_FR_60HZ,
		{1280, 48, 32, 80, 1440, 1,
		800, 3, 6, 14, 823, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1280x800_75_P, "1280x800@75Hz", XVIDC_FR_75HZ,
		{1280, 80, 128, 208, 1696, 0,
		800, 3, 6, 29, 838, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x800_85_P, "1280x800@85Hz", XVIDC_FR_85HZ,
		{1280, 80, 136, 216, 1712, 0,
		800, 3, 6, 34, 843, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x800_120_P_RB, "1280x800@120Hz (RB)", XVIDC_FR_120HZ,
		{1280, 48, 32, 80, 1440, 1,
		800, 3, 6, 38, 847, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1280x960_60_P, "1280x960@60Hz", XVIDC_FR_60HZ,
		{1280, 96, 112, 312, 1800, 1,
		960, 1, 3, 36, 1000, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x960_85_P, "1280x960@85Hz", XVIDC_FR_85HZ,
		{1280, 64, 160, 224, 1728, 1,
		960, 1, 3, 47, 1011, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x960_120_P_RB, "1280x960@120Hz (RB)", XVIDC_FR_120HZ,
		{1280, 48, 32, 80, 1440, 1,
		960, 3, 4, 50, 1017, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1280x1024_60_P, "1280x1024@60Hz", XVIDC_FR_60HZ,
		{1280, 48, 112, 248, 1688, 1,
		1024, 1, 3, 38, 1066, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x1024_75_P, "1280x1024@75Hz", XVIDC_FR_75HZ,
		{1280, 16, 144, 248, 1688, 1,
		1024, 1, 3, 38, 1066, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x1024_85_P, "1280x1024@85Hz", XVIDC_FR_85HZ,
		{1280, 64, 160, 224, 1728, 1,
		1024, 1, 3, 44, 1072, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1280x1024_120_P_RB, "1280x1024@120Hz (RB)", XVIDC_FR_120HZ,
		{1280, 48, 32, 80, 1440, 1,
		1024, 3, 7, 50, 1084, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1360x768_60_P, "1360x768@60Hz", XVIDC_FR_60HZ,
		{1360, 64, 112, 256, 1792, 1,
		768, 3, 6, 18, 795, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1360x768_120_P_RB, "1360x768@120Hz (RB)", XVIDC_FR_120HZ,
		{1360, 48, 32, 80, 1520, 1,
		768, 3, 5, 37, 813, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1366x768_60_P, "1366x768@60Hz", XVIDC_FR_60HZ,
		{1366, 70, 143, 213, 1792, 1,
		768, 3, 3, 24, 798, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1366x768_60_P_RB, "1366x768@60Hz (RB)", XVIDC_FR_60HZ,
		{1366, 14, 56, 64, 1500, 1,
		768, 1, 3, 28, 800, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1400x1050_60_P, "1400x1050@60Hz", XVIDC_FR_60HZ,
		{1400, 88, 144, 232, 1864, 0,
		1050, 3, 4, 32, 1089, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1400x1050_60_P_RB, "1400x1050@60Hz (RB)", XVIDC_FR_60HZ,
		{1400, 48, 32, 80, 1560, 1,
		1050, 3, 4, 23, 1080, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1400x1050_75_P, "1400x1050@75Hz", XVIDC_FR_75HZ,
		{1400, 104, 144, 248, 1896, 0,
		1050, 3, 4, 42, 1099, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1400x1050_85_P, "1400x1050@85Hz", XVIDC_FR_85HZ,
		{1400, 104, 152, 256, 1912, 0,
		1050, 3, 4, 48, 1105, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1400x1050_120_P_RB, "1400x1050@120Hz (RB)", XVIDC_FR_120HZ,
		{1400, 48, 32, 80, 1560, 1,
		1050, 3, 4, 55, 1112, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1440x240_60_P, "1440x240@60Hz", XVIDC_FR_60HZ,
		{1440, 38, 124, 114, 1716, 0,
		240, 14, 3, 4, 262, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1440x900_60_P, "1440x900@60Hz", XVIDC_FR_60HZ,
		{1440, 80, 152, 232, 1904, 0,
		900, 3, 6, 25, 934, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1440x900_60_P_RB, "1440x900@60Hz (RB)", XVIDC_FR_60HZ,
		{1440, 48, 32, 80, 1600, 1,
		900, 3, 6, 17, 926, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1440x900_75_P, "1440x900@75Hz", XVIDC_FR_75HZ,
		{1440, 96, 152, 248, 1936, 0,
		900, 3, 6, 33, 942, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1440x900_85_P, "1440x900@85Hz", XVIDC_FR_85HZ,
		{1440, 104, 152, 256, 1952, 0,
		900, 3, 6, 39, 948, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1440x900_120_P_RB, "1440x900@120Hz (RB)", XVIDC_FR_120HZ,
		{1440, 48, 32, 80, 1600, 1,
		900, 3, 6, 44, 953, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1600x1200_60_P, "1600x1200@60Hz", XVIDC_FR_60HZ,
		{1600, 64, 192, 304, 2160, 1,
		1200, 1, 3, 46, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1600x1200_65_P, "1600x1200@65Hz", XVIDC_FR_65HZ,
		{1600, 64, 192, 304, 2160, 1,
		1200, 1, 3, 46, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1600x1200_70_P, "1600x1200@70Hz", XVIDC_FR_70HZ,
		{1600, 64, 192, 304, 2160, 1,
		1200, 1, 3, 46, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1600x1200_75_P, "1600x1200@75Hz", XVIDC_FR_75HZ,
		{1600, 64, 192, 304, 2160, 1,
		1200, 1, 3, 46, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1600x1200_85_P, "1600x1200@85Hz", XVIDC_FR_85HZ,
		{1600, 64, 192, 304, 2160, 1,
		1200, 1, 3, 46, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1600x1200_120_P_RB, "1600x1200@120Hz (RB)", XVIDC_FR_120HZ,
		{1600, 48, 32, 80, 1760, 1,
		1200, 3, 4, 64, 1271, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1680x720_50_P, "1680x720@50Hz", XVIDC_FR_50HZ,
		{1680, 260, 40, 220, 2200, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x720_60_P, "1680x720@60Hz", XVIDC_FR_60HZ,
		{1680, 260, 40, 220, 2200, 1,
		720, 5, 5, 20, 750, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x720_100_P, "1680x720@100Hz", XVIDC_FR_100HZ,
		{1680, 60, 40, 220, 2000, 1,
		720, 5, 5, 95, 825, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x720_120_P, "1680x720@120Hz", XVIDC_FR_120HZ,
		{1680, 60, 40, 220, 2000, 1,
		720, 5, 5, 95, 825, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x1050_50_P, "1680x1050@50Hz", XVIDC_FR_50HZ,
		{1680, 88, 176, 264, 2208, 0,
		1050, 3, 6, 24, 1083, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x1050_60_P, "1680x1050@60Hz", XVIDC_FR_60HZ,
		{1680, 104, 176, 280, 2240, 0,
		1050, 3, 6, 30, 1089, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x1050_60_P_RB, "1680x1050@60Hz (RB)", XVIDC_FR_60HZ,
		{1680, 48, 32, 80, 1840, 1,
		1050, 3, 6, 21, 1080, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1680x1050_75_P, "1680x1050@75Hz", XVIDC_FR_75HZ,
		{1680, 120, 176, 296, 2272, 0,
		1050, 3, 6, 40, 1099, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x1050_85_P, "1680x1050@85Hz", XVIDC_FR_85HZ,
		{1680, 128, 176, 304, 2288, 0,
		1050, 3, 6, 46, 1105, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1680x1050_120_P_RB, "1680x1050@120Hz (RB)", XVIDC_FR_120HZ,
		{1680, 48, 32, 80, 1840, 1,
		1050, 3, 6, 53, 1112, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1792x1344_60_P, "1792x1344@60Hz", XVIDC_FR_60HZ,
		{1792, 128, 200, 328, 2448, 0,
		1344, 1, 3, 46, 1394, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1792x1344_75_P, "1792x1344@75Hz", XVIDC_FR_75HZ,
		{1792, 96, 216, 352, 2456, 0,
		1344, 1, 3, 69, 1417, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1792x1344_120_P_RB, "1792x1344@120Hz (RB)", XVIDC_FR_120HZ,
		{1792, 48, 32, 80, 1952, 1,
		1344, 3, 4, 72, 1423, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1856x1392_60_P, "1856x1392@60Hz", XVIDC_FR_60HZ,
		{1856, 96, 224, 352, 2528, 0,
		1392, 1, 3, 43, 1439, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1856x1392_75_P, "1856x1392@75Hz", XVIDC_FR_75HZ,
		{1856, 128, 224, 352, 2560, 0,
		1392, 1, 3, 104, 1500, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1856x1392_120_P_RB, "1856x1392@120Hz (RB)", XVIDC_FR_120HZ,
		{1856, 48, 32, 80, 2016, 1,
		1392, 3, 4, 75, 1474, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1920x1080_24_P, "1920x1080@24Hz", XVIDC_FR_24HZ,
		{1920, 638, 44, 148, 2750, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_25_P, "1920x1080@25Hz", XVIDC_FR_25HZ,
		{1920, 528, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_30_P, "1920x1080@30Hz", XVIDC_FR_30HZ,
		{1920, 88, 44, 148, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_48_P, "1920x1080@48Hz", XVIDC_FR_48HZ,
		{1920, 638, 44, 148, 2750, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_50_P, "1920x1080@50Hz", XVIDC_FR_50HZ,
		{1920, 528, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_60_P, "1920x1080@60Hz", XVIDC_FR_60HZ,
		{1920, 88, 44, 148, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_100_P, "1920x1080@100Hz", XVIDC_FR_100HZ,
		{1920, 528, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1080_120_P, "1920x1080@120Hz", XVIDC_FR_120HZ,
		{1920, 88, 44, 148, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1200_60_P, "1920x1200@60Hz", XVIDC_FR_60HZ,
		{1920, 136, 200, 336, 2592, 0,
		1200, 3, 6, 36, 1245, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1200_60_P_RB, "1920x1200@60Hz (RB)", XVIDC_FR_60HZ,
		{1920, 48, 32, 80, 2080, 1,
		1200, 3, 6, 26, 1235, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1920x1200_75_P, "1920x1200@75Hz", XVIDC_FR_75HZ,
		{1920, 136, 208, 344, 2608, 0,
		1200, 3, 6, 46, 1255, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1200_85_P, "1920x1200@85Hz", XVIDC_FR_85HZ,
		{1920, 144, 208, 352, 2624, 0,
		1200, 3, 6, 53, 1262, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1200_120_P_RB, "1920x1200@120Hz (RB)", XVIDC_FR_120HZ,
		{1920, 48, 32, 80, 2080, 1,
		1200, 3, 6, 62, 1271, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1920x1440_60_P, "1920x1440@60Hz", XVIDC_FR_60HZ,
		{1920, 128, 208, 344, 2600, 0,
		1440, 1, 3, 56, 1500, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1440_75_P, "1920x1440@75Hz", XVIDC_FR_75HZ,
		{1920, 144, 224, 352, 2640, 0,
		1440, 1, 3, 56, 1500, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_1920x1440_120_P_RB, "1920x1440@120Hz (RB)", XVIDC_FR_120HZ,
		{1920, 48, 32, 80, 2080, 1,
		1440, 3, 4, 78, 1525, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_1920x2160_60_P, "1920x2160@60Hz", XVIDC_FR_60HZ,
		{1920, 88, 44, 148, 2200, 1,
		2160, 20, 10, 60, 2250, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_2048x1080_24_P, "2048x1080@24Hz", XVIDC_FR_24HZ,
		{2048, 510, 44, 148, 2750, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_25_P, "2048x1080@25Hz", XVIDC_FR_25HZ,
		{2048, 400, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_30_P, "2048x1080@30Hz", XVIDC_FR_30HZ,
		{2048, 66, 20, 66, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_48_P, "2048x1080@48Hz", XVIDC_FR_48HZ,
		{2048, 510, 44, 148, 2750, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_50_P, "2048x1080@50Hz", XVIDC_FR_50HZ,
		{2048, 400, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_60_P, "2048x1080@60Hz", XVIDC_FR_60HZ,
		{2048, 88, 44, 20, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_100_P, "2048x1080@100Hz", XVIDC_FR_100HZ,
		{2048, 528, 44, 148, 2640, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2048x1080_120_P, "2048x1080@120Hz", XVIDC_FR_120HZ,
		{2048, 88, 44, 148, 2200, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1080_50_P, "2560x1080@50Hz", XVIDC_FR_50HZ,
		{2560, 548, 44, 148, 3300, 1,
		1080, 4, 5, 36, 1125, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1080_60_P, "2560x1080@60Hz", XVIDC_FR_60HZ,
		{2560, 248, 44, 148, 3000, 1,
		1080, 4, 5, 11, 1100, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1080_100_P, "2560x1080@100Hz", XVIDC_FR_100HZ,
		{2560, 218, 44, 148, 2970, 1,
		1080, 4, 5, 161, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1080_120_P, "2560x1080@120Hz", XVIDC_FR_120HZ,
		{2560, 548, 44, 148, 3300, 1,
		1080, 4, 5, 161, 1250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1600_60_P, "2560x1600@60Hz", XVIDC_FR_60HZ,
		{2560, 192, 280, 472, 3504, 0,
		1600, 3, 6, 49, 1658, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1600_60_P_RB, "2560x1600@60Hz (RB)", XVIDC_FR_60HZ,
		{2560, 48, 32, 80, 2720, 1,
		1600, 3, 6, 37, 1646, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_2560x1600_75_P, "2560x1600@75Hz", XVIDC_FR_75HZ,
		{2560, 208, 280, 488, 3536, 0,
		1600, 3, 6, 63, 1672, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1600_85_P, "2560x1600@85Hz", XVIDC_FR_85HZ,
		{2560, 208, 280, 488, 3536, 0,
		1600, 3, 6, 73, 1682, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_2560x1600_120_P_RB, "2560x1600@120Hz (RB)", XVIDC_FR_120HZ,
		{2560, 48, 32, 80, 2720, 1,
		1600, 3, 6, 85, 1694, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_3840x2160_24_P, "3840x2160@24Hz", XVIDC_FR_24HZ,
		{3840, 1276, 88, 296, 5500, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_25_P, "3840x2160@25Hz", XVIDC_FR_25HZ,
		{3840, 1056, 88, 296, 5280, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_30_P, "3840x2160@30Hz", XVIDC_FR_30HZ,
		{3840, 176, 88, 296, 4400, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_48_P, "3840x2160@48Hz", XVIDC_FR_48HZ,
		{3840, 1276, 88, 296, 5500, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_50_P, "3840x2160@50Hz", XVIDC_FR_50HZ,
		{3840, 1056, 88, 296, 5280, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_60_P, "3840x2160@60Hz", XVIDC_FR_60HZ,
		{3840, 176, 88, 296, 4400, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_3840x2160_60_P_RB, "3840x2160@60Hz (RB)", XVIDC_FR_60HZ,
		{3840, 48, 32, 80, 4000, 1,
		2160, 3, 5, 54, 2222, 0, 0, 0, 0, 0} },
	{ XVIDC_VM_4096x2160_24_P, "4096x2160@24Hz", XVIDC_FR_24HZ,
		{4096, 1020, 88, 296, 5500, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_25_P, "4096x2160@25Hz", XVIDC_FR_25HZ,
		{4096, 968, 88, 128, 5280, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_30_P, "4096x2160@30Hz", XVIDC_FR_30HZ,
		{4096, 88, 88, 128, 4400, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_48_P, "4096x2160@48Hz", XVIDC_FR_48HZ,
		{4096, 1020, 88, 296, 5500, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_50_P, "4096x2160@50Hz", XVIDC_FR_50HZ,
		{4096, 968, 88, 128, 5280, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_60_P, "4096x2160@60Hz", XVIDC_FR_60HZ,
		{4096, 88, 88, 128, 4400, 1,
		2160, 8, 10, 72, 2250, 0, 0, 0, 0, 1} },
	{ XVIDC_VM_4096x2160_60_P_RB, "4096x2160@60Hz (RB)", XVIDC_FR_60HZ,
		{4096, 8, 32, 40, 4176, 1,
		2160, 48, 8, 6, 2222, 0, 0, 0, 0, 0} },
};

const XVidC_VideoTimingMode *XVidC_CustomTimingModes = NULL;
int XVidC_NumCustomModes = 0;

/**************************** Function Prototypes *****************************/

static const XVidC_VideoTimingMode *XVidC_GetCustomVideoModeData(
		XVidC_VideoMode VmId);
static U8_T XVidC_IsVtmRb(const char *VideoModeStr, U8_T RbN);

/*************************** Function Definitions *****************************/

/******************************************************************************/
/**
 * This function registers a user-defined custom video mode timing table with
 * video_common. Functions which search the available video modes, or take VmId
 * as an input, will operate on or check the custom video mode timing table in
 * addition to the pre-defined video mode timing table (XVidC_VideoTimingModes).
 *
 * @param	CustomTable is a pointer to the user-defined custom vide mode
 *		timing table to register.
 * @param	NumElems is the number of video modes supported by CustomTable.
 *
 * @return
 *		- XST_SUCCESS if the custom table was successfully registered.
 *		- XST_FAILURE if an existing custom table is already present.
 *
 * @note	IDs in the custom table may not conflict with IDs reserved by
 *		the XVidC_VideoMode enum.
 *
*******************************************************************************/
U32_T XVidC_RegisterCustomTimingModes(const XVidC_VideoTimingMode *CustomTable,
		                            U16_T NumElems)
{
	U16_T Index;

	/* Verify arguments. */
	Xil_AssertNonvoid(CustomTable != NULL);
	for (Index = 0; Index < NumElems; Index++) {
		Xil_AssertNonvoid((CustomTable[Index].VmId > XVIDC_VM_CUSTOM));
		/* The IDs of each video mode in the custom table must not
		 * conflict with IDs reserved by video_common. */
	}

	/* Fail if a custom table is currently already registered. */
	if (XVidC_CustomTimingModes) {
		return XST_FAILURE;
	}

	XVidC_CustomTimingModes = CustomTable;
	XVidC_NumCustomModes    = NumElems;

	return XST_SUCCESS;
}

/******************************************************************************/
/**
 * This function unregisters the user-defined custom video mode timing table
 * previously registered by XVidC_RegisterCustomTimingModes().
 *
 * @return	None.
 *
 * @note	None.
 *
*******************************************************************************/
void XVidC_UnregisterCustomTimingModes(void)
{
	XVidC_CustomTimingModes = NULL;
	XVidC_NumCustomModes    = 0;
}

/******************************************************************************/
/**
 * This function calculates pixel clock based on the inputs.
 *
 * @param	HTotal specifies horizontal total.
 * @param	VTotal specifies vertical total.
 * @param	FrameRate specifies rate at which frames are generated.
 *
 * @return	Pixel clock in Hz.
 *
 * @note	None.
 *
*******************************************************************************/
U32_T XVidC_GetPixelClockHzByHVFr(U32_T HTotal, U32_T VTotal, U8_T FrameRate)
{
	return (HTotal * VTotal * FrameRate);
}

/******************************************************************************/
/**
 * This function calculates pixel clock from video mode.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Pixel clock in Hz.
 *
 * @note	None.
 *
*******************************************************************************/
U32_T XVidC_GetPixelClockHzByVmId(XVidC_VideoMode VmId)
{
	U32_T ClkHz;
	const XVidC_VideoTimingMode *VmPtr;

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return 0;
	}

	if (XVidC_IsInterlaced(VmId)) {
		/* For interlaced mode, use both frame 0 and frame 1 vertical
		 * totals. */
		ClkHz = VmPtr->Timing.F0PVTotal + VmPtr->Timing.F1VTotal;

		/* Multiply the number of pixels by the frame rate of each
		 * individual frame (half of the total frame rate). */
		ClkHz *= VmPtr->FrameRate / 2;
	}
	else {
		/* For progressive mode, use only frame 0 vertical total. */
		ClkHz = VmPtr->Timing.F0PVTotal;

		/* Multiply the number of pixels by the frame rate. */
		ClkHz *= VmPtr->FrameRate;
	}

	/* Multiply the vertical total by the horizontal total for number of
	 * pixels. */
	ClkHz *= VmPtr->Timing.HTotal;

	return ClkHz;
}

/******************************************************************************/
/**
 * This function checks if the input video mode is interlaced/progressive based
 * on its ID from the video timings table.
 *
 * @param	VmId specifies the resolution ID from the video timings table.
 *
 * @return	Video format.
 *		- XVIDC_VF_PROGRESSIVE
 *		- XVIDC_VF_INTERLACED
 *
 * @note	None.
 *
*******************************************************************************/
XVidC_VideoFormat XVidC_GetVideoFormat(XVidC_VideoMode VmId)
{
	const XVidC_VideoTimingMode *VmPtr;

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return XVIDC_VF_UNKNOWN;
	}

	if (VmPtr->Timing.F1VTotal == 0) {
		return (XVIDC_VF_PROGRESSIVE);
	}

	return (XVIDC_VF_INTERLACED);
}

/******************************************************************************/
/**
 * This function checks if the input video mode is interlaced based on its ID
 * from the video timings table.
 *
 * @param	VmId specifies the resolution ID from the video timings table.
 *
 * @return
 *		- 1 if the video timing with the supplied table ID is
 *		  interlaced.
 *		- 0 if the video timing is progressive.
 *
 * @note	None.
 *
*******************************************************************************/
U8_T XVidC_IsInterlaced(XVidC_VideoMode VmId)
{
	if (XVidC_GetVideoFormat(VmId) == XVIDC_VF_INTERLACED) {
		return 1;
	}

	return 0;
}

/******************************************************************************/
/**
 * This function returns the Video Mode ID that matches the detected input
 * timing, frame rate and I/P flag
 *
 * @param	Timing is the pointer to timing parameters to match
 * @param	FrameRate specifies refresh rate in HZ
 * @param	IsInterlaced is flag.
 *		      - 0 = Progressive
 *			  - 1 = Interlaced.
 *
 * @return	Id of a supported video mode.
 *
 * @note	This is an extension of XVidC_GetVideoModeId API to include
 *          blanking information in match process. No attempt is made to
 *          search for reduced blanking entries, if any.
 *
*******************************************************************************/
XVidC_VideoMode XVidC_GetVideoModeIdWBlanking(const XVidC_VideoTiming *Timing,
		                                      U32_T FrameRate, U8_T IsInterlaced)
{
  XVidC_VideoMode VmId;
  XVidC_VideoTiming const *StdTiming = NULL;

  /* First search for ID with matching Width & Height */
  VmId = XVidC_GetVideoModeId(Timing->HActive, Timing->VActive, FrameRate,
		                      IsInterlaced);

  if(VmId == XVIDC_VM_NOT_SUPPORTED) {
	return(VmId);
  } else {

	/* Get standard timing info from default timing table */
	StdTiming = XVidC_GetTimingInfo(VmId);

	/* Match against detected timing parameters */
	if((Timing->HActive        == StdTiming->HActive) &&
	   (Timing->VActive        == StdTiming->VActive) &&
	   (Timing->HTotal         == StdTiming->HTotal) &&
	   (Timing->F0PVTotal      == StdTiming->F0PVTotal) &&
	   (Timing->HFrontPorch    == StdTiming->HFrontPorch) &&
	   (Timing->HSyncWidth     == StdTiming->HSyncWidth) &&
	   (Timing->HBackPorch     == StdTiming->HBackPorch) &&
	   (Timing->F0PVFrontPorch == StdTiming->F0PVFrontPorch) &&
	   (Timing->F0PVSyncWidth  == StdTiming->F0PVSyncWidth) &&
	   (Timing->F0PVBackPorch  == StdTiming->F0PVBackPorch)) {
       return(VmId);
	 } else {
	   return(XVIDC_VM_NOT_SUPPORTED);
	 }
  }
}

/******************************************************************************/
/**
 * This function returns the Video Mode ID that matches the detected input
 * width, height, frame rate and I/P flag
 *
 * @param	Width specifies the number pixels per scanline.
 * @param	Height specifies the number of scanline's.
 * @param	FrameRate specifies refresh rate in HZ
 * @param	IsInterlaced is flag.
 *		- 0 = Progressive
 *		- 1 = Interlaced.
 *
 * @return	Id of a supported video mode.
 *
 * @note	None.
 *
*******************************************************************************/
XVidC_VideoMode XVidC_GetVideoModeId(U32_T Width, U32_T Height, U32_T FrameRate,
					U8_T IsInterlaced)
{
	U32_T Low;
	U32_T High;
	U32_T Mid;
	U32_T HActive;
	U32_T VActive;
	U32_T Rate;
	U32_T ResFound = (AVER_FALSE);
	XVidC_VideoMode Mode;
	U16_T Index;

	/* First, attempt a linear search on the custom video timing table. */
	if(XVidC_CustomTimingModes) {
	  for (Index = 0; Index < XVidC_NumCustomModes; Index++) {
		HActive = XVidC_CustomTimingModes[Index].Timing.HActive;
		VActive = XVidC_CustomTimingModes[Index].Timing.VActive;
		Rate = XVidC_CustomTimingModes[Index].FrameRate;
		if ((Width  == HActive) &&
			(Height == VActive) &&
			(FrameRate == Rate)) {
			   return XVidC_CustomTimingModes[Index].VmId;
		}
	  }
	}

	if (IsInterlaced) {
		Low = (XVIDC_VM_INTL_START);
		High = (XVIDC_VM_INTL_END);
	}
	else {
		Low = (XVIDC_VM_PROG_START);
		High = (XVIDC_VM_PROG_END);
	}

	HActive = VActive = Rate = 0;

	/* Binary search finds item in sorted array.
	 * And returns index (zero based) of item
	 * If item is not found returns flag remains
	 * AVER_FALSE. Search key is "width or HActive"
	 */
	while (Low <= High) {
		Mid = (Low + High) / 2;
		HActive = XVidC_VideoTimingModes[Mid].Timing.HActive;
		if (Width == HActive) {
			ResFound = (AVER_TRUE);
			break;
		}
		else if (Width < HActive) {
			if (Mid == 0) {
				break;
			}
			else {
				High = Mid - 1;
			}
		}
		else {
			Low = Mid + 1;
		}
	}

	 /* HActive matched at middle */
	if (ResFound) {
		/* Rewind to start index of mode with matching width */
		while ((Mid > 0) &&
			(XVidC_VideoTimingModes[Mid - 1].Timing.HActive ==
								Width)) {
			--Mid;
		}

		ResFound = (AVER_FALSE);
		VActive = XVidC_VideoTimingModes[Mid].Timing.VActive;
		Rate = XVidC_VideoTimingModes[Mid].FrameRate;

		/* Now do a linear search for matching VActive and Frame
		 * Rate
		 */
		while (HActive == Width) {
			/* check current entry */
			if ((VActive == Height) && (Rate == FrameRate)) {
				ResFound = (AVER_TRUE);
				break;
			}
			/* Check next entry */
			else {
				Mid = Mid + 1;
				HActive =
				XVidC_VideoTimingModes[Mid].Timing.HActive;
				VActive =
				XVidC_VideoTimingModes[Mid].Timing.VActive;
				Rate = XVidC_VideoTimingModes[Mid].FrameRate;
			}
		}
		Mode =
		(ResFound) ? (XVidC_VideoMode)Mid : (XVIDC_VM_NOT_SUPPORTED);
	}
	else {
		Mode = (XVIDC_VM_NOT_SUPPORTED);
	}

	return (Mode);
}

/******************************************************************************/
/**
 * This function returns the Video Mode ID that matches the detected input
 * timing, frame rate and I/P flag
 *
 * @param	Timing is the pointer to timing parameters to match
 * @param	FrameRate specifies refresh rate in HZ
 * @param	IsInterlaced is flag.
 *		      - 0 = Progressive
 *			  - 1 = Interlaced.
 * @param	IsExtensive is flag.
 *		      - 0 = Basic matching of timing parameters
 *			  - 1 = Extensive matching of timing parameters
 *
 * @return	Id of a supported video mode.
 *
 * @note	This function attempts to search for reduced blanking entries, if
 *          any.
 *
*******************************************************************************/
XVidC_VideoMode XVidC_GetVideoModeIdExtensive(XVidC_VideoTiming *Timing,
											  U32_T FrameRate,
											  U8_T IsInterlaced,
											  U8_T IsExtensive)
{
	U32_T Low;
	U32_T High;
	U32_T Mid;
	U32_T HActive;
	U32_T VActive;
	U32_T Rate;
	U32_T ResFound = (AVER_FALSE);
	XVidC_VideoMode Mode;
	U16_T Index;

	/* First, attempt a linear search on the custom video timing table. */
	if(XVidC_CustomTimingModes) {
	  for (Index = 0; Index < XVidC_NumCustomModes; Index++) {
		HActive = XVidC_CustomTimingModes[Index].Timing.HActive;
		VActive = XVidC_CustomTimingModes[Index].Timing.VActive;
		Rate = XVidC_CustomTimingModes[Index].FrameRate;
		if ((HActive == Timing->HActive) && (VActive == Timing->VActive) &&
				(Rate == FrameRate) && (IsExtensive == 0 || (
			XVidC_CustomTimingModes[Index].Timing.HTotal == Timing->HTotal &&
			XVidC_CustomTimingModes[Index].Timing.F0PVTotal ==
					Timing->F0PVTotal &&
			XVidC_CustomTimingModes[Index].Timing.HFrontPorch ==
					Timing->HFrontPorch &&
			XVidC_CustomTimingModes[Index].Timing.F0PVFrontPorch ==
					Timing->F0PVFrontPorch &&
			XVidC_CustomTimingModes[Index].Timing.HSyncWidth ==
					Timing->HSyncWidth &&
			XVidC_CustomTimingModes[Index].Timing.F0PVSyncWidth ==
					Timing->F0PVSyncWidth &&
			XVidC_CustomTimingModes[Index].Timing.VSyncPolarity ==
					Timing->VSyncPolarity))) {
				if (!IsInterlaced || IsExtensive == 0 || (
						XVidC_CustomTimingModes[Index].Timing.F1VTotal ==
						Timing->F1VTotal &&
						XVidC_CustomTimingModes[Index].Timing.F1VFrontPorch ==
								Timing->F1VFrontPorch &&
						XVidC_CustomTimingModes[Index].Timing.F1VSyncWidth ==
								Timing->F1VSyncWidth)) {
					return XVidC_CustomTimingModes[Index].VmId;
				}
		}
	  }
	}

	if (IsInterlaced) {
		Low = (XVIDC_VM_INTL_START);
		High = (XVIDC_VM_INTL_END);
	}
	else {
		Low = (XVIDC_VM_PROG_START);
		High = (XVIDC_VM_PROG_END);
	}

	HActive = VActive = Rate = 0;

	/* Binary search finds item in sorted array.
	 * And returns index (zero based) of item
	 * If item is not found returns flag remains
	 * AVER_FALSE. Search key is "Timing->HActive or HActive"
	 */
	while (Low <= High) {
		Mid = (Low + High) / 2;
		HActive = XVidC_VideoTimingModes[Mid].Timing.HActive;
		if (Timing->HActive == HActive) {
			ResFound = (AVER_TRUE);
			break;
		}
		else if (Timing->HActive < HActive) {
			if (Mid == 0) {
				break;
			}
			else {
				High = Mid - 1;
			}
		}
		else {
			Low = Mid + 1;
		}
	}

	 /* HActive matched at middle */
	if (ResFound) {
		/* Rewind to start index of mode with matching Timing->HActive */
		while ((Mid > 0) &&
			(XVidC_VideoTimingModes[Mid - 1].Timing.HActive ==
								Timing->HActive)) {
			--Mid;
		}

		ResFound = (AVER_FALSE);
		VActive = XVidC_VideoTimingModes[Mid].Timing.VActive;
		Rate = XVidC_VideoTimingModes[Mid].FrameRate;

		/* Now do a linear search for matching VActive and Frame
		 * Rate
		 */
		while (HActive == Timing->HActive) {
			/* check current entry */
			if ((VActive == Timing->VActive) && (Rate == FrameRate) &&
					(IsExtensive == 0 ||
					(XVidC_VideoTimingModes[Mid].Timing.HTotal ==
							Timing->HTotal &&
					XVidC_VideoTimingModes[Mid].Timing.F0PVTotal ==
							Timing->F0PVTotal &&
					XVidC_VideoTimingModes[Mid].Timing.HFrontPorch ==
							Timing->HFrontPorch &&
					XVidC_VideoTimingModes[Mid].Timing.F0PVFrontPorch ==
							Timing->F0PVFrontPorch &&
					XVidC_VideoTimingModes[Mid].Timing.HSyncWidth ==
							Timing->HSyncWidth &&
					XVidC_VideoTimingModes[Mid].Timing.F0PVSyncWidth ==
							Timing->F0PVSyncWidth &&
					XVidC_VideoTimingModes[Mid].Timing.VSyncPolarity ==
							Timing->VSyncPolarity))) {
				if (!IsInterlaced || IsExtensive == 0 || (
						XVidC_VideoTimingModes[Mid].Timing.F1VTotal ==
								Timing->F1VTotal &&
						XVidC_VideoTimingModes[Mid].Timing.F1VFrontPorch ==
								Timing->F1VFrontPorch &&
						XVidC_VideoTimingModes[Mid].Timing.F1VSyncWidth ==
								Timing->F1VSyncWidth)) {
					ResFound = (AVER_TRUE);
					break;
				}
			}
			/* Check next entry */
			else {
				Mid = Mid + 1;
				HActive =
				XVidC_VideoTimingModes[Mid].Timing.HActive;
				VActive =
				XVidC_VideoTimingModes[Mid].Timing.VActive;
				Rate = XVidC_VideoTimingModes[Mid].FrameRate;
			}
		}
		Mode =
		(ResFound) ? (XVidC_VideoMode)Mid : (XVIDC_VM_NOT_SUPPORTED);
	}
	else {
		Mode = (XVIDC_VM_NOT_SUPPORTED);
	}

	return (Mode);
}

/******************************************************************************/
/**
 * This function returns the video mode ID that matches the detected input
 * width, height, frame rate, interlaced or progressive, and reduced blanking.
 *
 * @param	Width specifies the number pixels per scanline.
 * @param	Height specifies the number of scanline's.
 * @param	FrameRate specifies refresh rate in HZ
 * @param	IsInterlaced specifies interlaced or progressive mode:
 *		- 0 = Progressive
 *		- 1 = Interlaced.
 * @param	RbN specifies the type of reduced blanking:
 *		- 0 = No reduced blanking
 *		- 1 = RB
 *		- 2 = RB2
 *
 * @return	ID of a supported video mode.
 *
 * @note	None.
 *
*******************************************************************************/
XVidC_VideoMode XVidC_GetVideoModeIdRb(U32_T Width, U32_T Height,
		U32_T FrameRate, U8_T IsInterlaced, U8_T RbN)
{
	XVidC_VideoMode VmId;
	const XVidC_VideoTimingMode *VtmPtr;
	U8_T Found = 0;

	VmId = XVidC_GetVideoModeId(Width, Height, FrameRate,
				IsInterlaced);

	VtmPtr = XVidC_GetVideoModeData(VmId);
	if (!VtmPtr) {
		return XVIDC_VM_NOT_SUPPORTED;
	}

	while (!Found) {
		VtmPtr = XVidC_GetVideoModeData(VmId);
		if ((Height != VtmPtr->Timing.VActive) ||
		    (Width != VtmPtr->Timing.HActive) ||
		    (FrameRate != VtmPtr->FrameRate) ||
		    (IsInterlaced && !XVidC_IsInterlaced(VmId))) {
			VmId = XVIDC_VM_NOT_SUPPORTED;
			break;
		}
		Found = XVidC_IsVtmRb(XVidC_GetVideoModeStr(VmId), RbN);
		if (Found) {
			break;
		}
		VmId = (XVidC_VideoMode)((int)VmId + 1);
	}

	return VmId;
}

/******************************************************************************/
/**
 * This function returns the pointer to video mode data at index provided.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Pointer to XVidC_VideoTimingMode structure based on the given
 *		video mode.
 *
 * @note	None.
 *
*******************************************************************************/
const XVidC_VideoTimingMode *XVidC_GetVideoModeData(XVidC_VideoMode VmId)
{
	if (VmId < XVIDC_VM_NUM_SUPPORTED) {
		return &XVidC_VideoTimingModes[VmId];
	}

	return XVidC_GetCustomVideoModeData(VmId);
}

/******************************************************************************/
/**
 *
 * This function returns the resolution name for index specified.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Pointer to a resolution name string.
 *
 * @note	None.
 *
*******************************************************************************/
const char *XVidC_GetVideoModeStr(XVidC_VideoMode VmId)
{
	const XVidC_VideoTimingMode *VmPtr;

	if (VmId == XVIDC_VM_CUSTOM) {
		return ("Custom video mode");
	}

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return ("Video mode not supported");
	}

	return VmPtr->Name;
}

/******************************************************************************/
/**
 * This function returns the frame rate name for index specified.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Pointer to a frame rate name string.
 *
 * @note	None.
 *
*******************************************************************************/
const char *XVidC_GetFrameRateStr(XVidC_VideoMode VmId)
{
	const XVidC_VideoTimingMode *VmPtr;

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return ("Video mode not supported");
	}

	switch (VmPtr->FrameRate) {
		case (XVIDC_FR_24HZ):   return ("24Hz");
		case (XVIDC_FR_25HZ):   return ("25Hz");
		case (XVIDC_FR_30HZ):   return ("30Hz");
		case (XVIDC_FR_48HZ):   return ("48Hz");
		case (XVIDC_FR_50HZ):   return ("50Hz");
		case (XVIDC_FR_56HZ):   return ("56Hz");
		case (XVIDC_FR_60HZ):   return ("60Hz");
		case (XVIDC_FR_65HZ):   return ("65Hz");
		case (XVIDC_FR_67HZ):   return ("67Hz");
		case (XVIDC_FR_70HZ):   return ("70Hz");
		case (XVIDC_FR_72HZ):   return ("72Hz");
		case (XVIDC_FR_75HZ):   return ("75Hz");
		case (XVIDC_FR_85HZ):   return ("85Hz");
		case (XVIDC_FR_87HZ):   return ("87Hz");
		case (XVIDC_FR_88HZ):   return ("88Hz");
		case (XVIDC_FR_96HZ):   return ("96Hz");
		case (XVIDC_FR_100HZ):  return ("100Hz");
		case (XVIDC_FR_120HZ):  return ("120Hz");

		default:
		     return ("Frame rate not supported");
	}
}

/******************************************************************************/
/**
 * This function returns a string representation of the enumerated type,
 * XVidC_3DFormat.
 *
 * @param	Format specifies the value to convert.
 *
 * @return	Pointer to the converted string.
 *
 * @note	None.
 *
*******************************************************************************/
const char *XVidC_Get3DFormatStr(XVidC_3DFormat Format)
{
	switch (Format) {
		case XVIDC_3D_FRAME_PACKING:
			return ("Frame Packing");

		case XVIDC_3D_FIELD_ALTERNATIVE:
			return ("Field Alternative");

		case XVIDC_3D_LINE_ALTERNATIVE:
			return ("Line Alternative");

		case XVIDC_3D_SIDE_BY_SIDE_FULL:
			return ("Side-by-Side(full)");

		case XVIDC_3D_TOP_AND_BOTTOM_HALF:
			return ("Top-and-Bottom(half)");

		case XVIDC_3D_SIDE_BY_SIDE_HALF:
			return ("Side-by-Side(half)");

		default:
			return ("Unknown");
	}
}

/******************************************************************************/
/**
 * This function returns the color format name for index specified.
 *
 * @param	ColorFormatId specifies the index of color format space.
 *
 * @return	Pointer to a color space name string.
 *
 * @note	None.
 *
*******************************************************************************/
const char *XVidC_GetColorFormatStr(XVidC_ColorFormat ColorFormatId)
{
	switch (ColorFormatId) {
		case XVIDC_CSF_RGB:            return ("RGB");
		case XVIDC_CSF_YCRCB_444:      return ("YUV_444");
		case XVIDC_CSF_YCRCB_422:      return ("YUV_422");
		case XVIDC_CSF_YCRCB_420:      return ("YUV_420");
		case XVIDC_CSF_YONLY:          return ("Y_ONLY");
		case XVIDC_CSF_RGBA:           return ("RGBA");
		case XVIDC_CSF_YCRCBA_444:     return ("YUVA_444");
		case XVIDC_CSF_MEM_RGBX8:      return ("RGBX8");
		case XVIDC_CSF_MEM_YUVX8:      return ("YUVX8");
		case XVIDC_CSF_MEM_YUYV8:      return ("YUYV8");
		case XVIDC_CSF_MEM_RGBA8:      return ("RGBA8");
		case XVIDC_CSF_MEM_YUVA8:      return ("YUVA8");
		case XVIDC_CSF_MEM_RGBX10:     return ("RGBX10");
		case XVIDC_CSF_MEM_YUVX10:     return ("YUVX10");
		case XVIDC_CSF_MEM_RGB565:     return ("RGB565");
		case XVIDC_CSF_MEM_Y_UV8:      return ("Y_UV8");
		case XVIDC_CSF_MEM_Y_UV8_420:  return ("Y_UV8_420");
		case XVIDC_CSF_MEM_RGB8:       return ("RGB8");
		case XVIDC_CSF_MEM_YUV8:       return ("YUV8");
		case XVIDC_CSF_MEM_Y_UV10:     return ("Y_UV10");
		case XVIDC_CSF_MEM_Y_UV10_420: return ("Y_UV10_420");
		case XVIDC_CSF_MEM_Y8:         return ("Y8");
		case XVIDC_CSF_MEM_Y10:        return ("Y10");
		case XVIDC_CSF_MEM_BGRA8:      return ("BGRA8");
		case XVIDC_CSF_MEM_BGRX8:      return ("BGRX8");
		case XVIDC_CSF_MEM_UYVY8:      return ("UYVY8");
		case XVIDC_CSF_MEM_BGR8:       return ("BGR8");
		case XVIDC_CSF_YCBCR_422:      return ("YCBCR_422");
		case XVIDC_CSF_YCBCR_420:      return ("YCBCR_420");
		default:
					       return ("Color space format not supported");
	}
}

/******************************************************************************/
/**
 * This function returns the frame rate for index specified.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Frame rate in Hz.
 *
 * @note	None.
 *
*******************************************************************************/
XVidC_FrameRate XVidC_GetFrameRate(XVidC_VideoMode VmId)
{
	const XVidC_VideoTimingMode *VmPtr;

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return XVIDC_FR_NUM_SUPPORTED;
	}

	return VmPtr->FrameRate;
}

/******************************************************************************/
/**
 * This function returns the timing parameters for specified resolution.
 *
 * @param	VmId specifies the resolution id.
 *
 * @return	Pointer to a XVidC_VideoTiming structure.
 *
 * @note	None.
 *
*******************************************************************************/
const XVidC_VideoTiming *XVidC_GetTimingInfo(XVidC_VideoMode VmId)
{
	const XVidC_VideoTimingMode *VmPtr;

	VmPtr = XVidC_GetVideoModeData(VmId);
	if (!VmPtr) {
		return NULL;
	}

	return &VmPtr->Timing;
}

/******************************************************************************/
/**
 * This function sets the VideoStream structure for the specified video format.
 *
 * @param	VidStrmPtr is a pointer to the XVidC_VideoStream structure to be
 *		set.
 * @param	VmId specifies the resolution ID.
 * @param	ColorFormat specifies the color format type.
 * @param	Bpc specifies the color depth/bits per color component.
 * @param	Ppc specifies the pixels per clock.
 *
 * @return
 *		- XST_SUCCESS if the timing for the supplied ID was found.
 *		- XST_FAILURE, otherwise.
 *
 * @note	None.
 *
*******************************************************************************/
U32_T XVidC_SetVideoStream(XVidC_VideoStream *VidStrmPtr, XVidC_VideoMode VmId,
			 XVidC_ColorFormat ColorFormat, XVidC_ColorDepth Bpc,
			 XVidC_PixelsPerClock Ppc)
{
	const XVidC_VideoTiming *TimingPtr;

	/* Verify arguments. */
	Xil_AssertNonvoid(VidStrmPtr != NULL);
	Xil_AssertNonvoid(ColorFormat < XVIDC_CSF_NUM_SUPPORTED);
	Xil_AssertNonvoid((Bpc == XVIDC_BPC_6)  ||
			  (Bpc == XVIDC_BPC_8)  ||
			  (Bpc == XVIDC_BPC_10) ||
			  (Bpc == XVIDC_BPC_12) ||
			  (Bpc == XVIDC_BPC_14) ||
			  (Bpc == XVIDC_BPC_16) ||
			  (Bpc == XVIDC_BPC_UNKNOWN));
	Xil_AssertNonvoid((Ppc == XVIDC_PPC_1) ||
			  (Ppc == XVIDC_PPC_2) ||
			  (Ppc == XVIDC_PPC_4));

	/* Get the timing from the video timing table. */
	TimingPtr = XVidC_GetTimingInfo(VmId);
	if (!TimingPtr) {
		return XST_FAILURE;
	}
	VidStrmPtr->VmId		= VmId;
	VidStrmPtr->Timing		= *TimingPtr;
	VidStrmPtr->FrameRate		= XVidC_GetFrameRate(VmId);
	VidStrmPtr->IsInterlaced	= XVidC_IsInterlaced(VmId);
	VidStrmPtr->ColorFormatId	= ColorFormat;
	VidStrmPtr->ColorDepth		= Bpc;
	VidStrmPtr->PixPerClk		= Ppc;

	/* Set stream to 2D. */
	VidStrmPtr->Is3D			= AVER_FALSE;
	VidStrmPtr->Info_3D.Format		= XVIDC_3D_UNKNOWN;
	VidStrmPtr->Info_3D.Sampling.Method	= XVIDC_3D_SAMPLING_UNKNOWN;
	VidStrmPtr->Info_3D.Sampling.Position	= XVIDC_3D_SAMPPOS_UNKNOWN;

	return XST_SUCCESS;
}

/******************************************************************************/
/**
 * This function sets the VideoStream structure for the specified 3D video
 * format.
 *
 * @param	VidStrmPtr is a pointer to the XVidC_VideoStream structure to be
 *		set.
 * @param	VmId specifies the resolution ID.
 * @param	ColorFormat specifies the color format type.
 * @param	Bpc specifies the color depth/bits per color component.
 * @param	Ppc specifies the pixels per clock.
 * @param	Info3DPtr is a pointer to a XVidC_3DInfo structure.
 *
 * @return
 *		- XST_SUCCESS if the timing for the supplied ID was found.
 *		- XST_FAILURE, otherwise.
 *
 * @return
 *		- XST_SUCCESS
 *		- XST_FAILURE
 *
 * @note	None.
 *
*******************************************************************************/
U32_T XVidC_Set3DVideoStream(XVidC_VideoStream *VidStrmPtr, XVidC_VideoMode VmId,
			   XVidC_ColorFormat ColorFormat, XVidC_ColorDepth Bpc,
			   XVidC_PixelsPerClock Ppc, XVidC_3DInfo *Info3DPtr)
{
	U32_T Status;
	U16_T Vblank0;
	U16_T Vblank1;

	/* Verify arguments */
	Xil_AssertNonvoid(Info3DPtr != NULL);

	/* Initialize with info for 2D frame. */
	Status = XVidC_SetVideoStream(VidStrmPtr, VmId, ColorFormat, Bpc, Ppc);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/* Set stream to 3D. */
	VidStrmPtr->Is3D	= AVER_TRUE;
	VidStrmPtr->Info_3D	= *Info3DPtr;

	/* Only 3D format supported is frame packing. */
	if (Info3DPtr->Format != XVIDC_3D_FRAME_PACKING) {
		return XST_FAILURE;
	}

	/* Update the timing based on the 3D format. */

	/* An interlaced format is converted to a progressive frame: */
	/*	3D VActive = (2D VActive * 4) + (2D VBlank field0) +
						(2D Vblank field1 * 2) */
	if (VidStrmPtr->IsInterlaced) {
		Vblank0 = VidStrmPtr->Timing.F0PVTotal -
						VidStrmPtr->Timing.VActive;
		Vblank1 = VidStrmPtr->Timing.F1VTotal -
						VidStrmPtr->Timing.VActive;
		VidStrmPtr->Timing.VActive = (VidStrmPtr->Timing.VActive * 4) +
						Vblank0 + (Vblank1 * 2);

		/* Set VTotal */
		VidStrmPtr->Timing.F0PVTotal *= 2;
		VidStrmPtr->Timing.F0PVTotal += VidStrmPtr->Timing.F1VTotal * 2;

		/* Clear field 1 values. */
		VidStrmPtr->Timing.F1VFrontPorch = 0;
		VidStrmPtr->Timing.F1VSyncWidth  = 0;
		VidStrmPtr->Timing.F1VBackPorch  = 0;
		VidStrmPtr->Timing.F1VTotal      = 0;

		/* Set format to progressive */
		VidStrmPtr->IsInterlaced = AVER_FALSE;
	}
	/* Progressive */
	else {
		/* 3D Vactive = (2D VActive * 2) + (2D VBlank) */
		Vblank0 = VidStrmPtr->Timing.F0PVTotal -
						VidStrmPtr->Timing.VActive;
		VidStrmPtr->Timing.VActive = (VidStrmPtr->Timing.VActive * 2) +
						Vblank0;

		/* Set VTotal. */
		VidStrmPtr->Timing.F0PVTotal = VidStrmPtr->Timing.F0PVTotal * 2;
	}

	return XST_SUCCESS;
}

/******************************************************************************/
/**
 * This function prints the stream information on STDIO/UART console.
 *
 * @param	Stream is a pointer to video stream.
 *
 * @return	None.
 *
 * @note	None.
 *
*******************************************************************************/
void XVidC_ReportStreamInfo(const XVidC_VideoStream *Stream)
{
	if (!XVidC_GetVideoModeData(Stream->VmId) &&
			(Stream->VmId != XVIDC_VM_CUSTOM)) {
		mesg("\tThe stream ID (%d) is not supported.\r\n",
				Stream->VmId);
		return;
	}

	mesg("\tColor Format:     %s\r\n",
			XVidC_GetColorFormatStr(Stream->ColorFormatId));
	mesg("\tColor Depth:      %d\r\n", Stream->ColorDepth);
	mesg("\tPixels Per Clock: %d\r\n", Stream->PixPerClk);
	mesg("\tMode:             %s\r\n",
			Stream->IsInterlaced ? "Interlaced" : "Progressive");

	if (Stream->Is3D) {
		mesg("\t3D Format:        %s\r\n",
		XVidC_Get3DFormatStr(Stream->Info_3D.Format));
	}

	if (Stream->VmId == XVIDC_VM_CUSTOM) {
		mesg("\tFrame Rate:       %dHz\r\n",
				Stream->FrameRate);
		mesg("\tResolution:       %dx%d [Custom Mode]\r\n",
				Stream->Timing.HActive, Stream->Timing.VActive);
		mesg("\tPixel Clock:      %d\r\n",
				XVidC_GetPixelClockHzByHVFr(
					Stream->Timing.HTotal,
					Stream->Timing.F0PVTotal,
					Stream->FrameRate));
	}
	else {
		mesg("\tFrame Rate:       %s\r\n",
				XVidC_GetFrameRateStr(Stream->VmId));
		mesg("\tResolution:       %s\r\n",
				XVidC_GetVideoModeStr(Stream->VmId));
		mesg("\tPixel Clock:      %d\r\n",
				XVidC_GetPixelClockHzByVmId(Stream->VmId));
	}
}

/******************************************************************************/
/**
 * This function prints timing information on STDIO/Uart console.
 *
 * @param	Timing is a pointer to Video Timing structure of the stream.
 * @param	IsInterlaced is a AVER_TRUE/AVER_FALSE flag that denotes the timing
 *		parameter is for interlaced/progressive stream.
 *
 * @return	None.
 *
 * @note	None.
 *
*******************************************************************************/
void XVidC_ReportTiming(const XVidC_VideoTiming *Timing, U8_T IsInterlaced)
{
	mesg("\r\n\tHSYNC Timing: hav=%04d, hfp=%02d, hsw=%02d(hsp=%d), "
			"hbp=%03d, htot=%04d \n\r", Timing->HActive,
			Timing->HFrontPorch, Timing->HSyncWidth,
			Timing->HSyncPolarity,
			Timing->HBackPorch, Timing->HTotal);

	/* Interlaced */
	if (IsInterlaced) {
		mesg("\tVSYNC Timing (Field 0): vav=%04d, vfp=%02d, "
			"vsw=%02d(vsp=%d), vbp=%03d, vtot=%04d\n\r",
			Timing->VActive, Timing->F0PVFrontPorch,
			Timing->F0PVSyncWidth, Timing->VSyncPolarity,
			Timing->F0PVBackPorch, Timing->F0PVTotal);
	mesg("\tVSYNC Timing (Field 1): vav=%04d, vfp=%02d, "
			"vsw=%02d(vsp=%d), vbp=%03d, vtot=%04d\n\r",
			Timing->VActive, Timing->F1VFrontPorch,
			Timing->F1VSyncWidth, Timing->VSyncPolarity,
			Timing->F1VBackPorch, Timing->F1VTotal);
	}
	/* Progressive */
	else {
		mesg("\tVSYNC Timing: vav=%04d, vfp=%02d, "
			"vsw=%02d(vsp=%d), vbp=%03d, vtot=%04d\n\r",
			Timing->VActive, Timing->F0PVFrontPorch,
			Timing->F0PVSyncWidth, Timing->VSyncPolarity,
			Timing->F0PVBackPorch, Timing->F0PVTotal);
	}
}

/******************************************************************************/
/**
 * This function returns the pointer to video mode data at the provided index
 * of the custom video mode table.
 *
 * @param	VmId specifies the resolution ID.
 *
 * @return	Pointer to XVidC_VideoTimingMode structure based on the given
 *		video mode.
 *
 * @note	None.
 *
*******************************************************************************/
static const XVidC_VideoTimingMode *XVidC_GetCustomVideoModeData(
		XVidC_VideoMode VmId)
{
	U16_T Index;

	for (Index = 0; Index < XVidC_NumCustomModes; Index++) {
		if (VmId == (XVidC_CustomTimingModes[Index].VmId)) {
			return &(XVidC_CustomTimingModes[Index]);
		}
	}

	/* ID not found within the custom video mode table. */
	return NULL;
}

/******************************************************************************/
/**
 * This function returns whether or not the video timing mode is a reduced
 * blanking mode or not.
 *
 * @param	VideoModeStr specifies the resolution name string.
 * @param	RbN specifies the type of reduced blanking:
 *		- 0 = No Reduced Blanking
 *		- 1 = RB
 *		- 2 = RB2
 *
 * @return	If the reduced blanking type is compatible with the video mode:
 *		- 0 = Not supported
 *		- 1 = Video mode supports the RB type
 *
 * @note	None.
 *
*******************************************************************************/
static U8_T XVidC_IsVtmRb(const char *VideoModeStr, U8_T RbN)
{
	while ((*VideoModeStr !='\0') && (*VideoModeStr != 'R')) {
		VideoModeStr++;
	}

	if (*(VideoModeStr + 2) == ')') {
		return RbN == 1;
	}
	if (*(VideoModeStr + 2) == '2') {
		return RbN == 2;
	}
	return 0;
}
/** @} */
